@@ -1,4 +1,4 @@
-from bs4 import BeautifulSoup, NavigableString, Comment
+from bs4 import BeautifulSoup, NavigableString, Comment, Doctype
 import re
 import six
 
@@ -25,10 +25,12 @@ ASTERISK = '*'
 UNDERSCORE = '_'
 
 
-def escape(text):
+def escape(text, escape_underscores):
     if not text:
         return ''
-    return text.replace('_', r'\_')
+    if escape_underscores:
+        return text.replace('_', r'\_')
+    return text
 
 
 def chomp(text):
@@ -66,13 +68,18 @@ def _todict(obj):
 
 class MarkdownConverter(object):
     class DefaultOptions:
-        strip = None
-        convert = None
         autolinks = True
-        heading_style = UNDERLINED
         bullets = '*+-'  # An iterable of bullet types.
-        strong_em_symbol = ASTERISK
+        code_language = ''
+        convert = None
+        default_title = False
+        escape_underscores = True
+        heading_style = UNDERLINED
         newline_style = SPACES
+        strip = None
+        strong_em_symbol = ASTERISK
+        sub_symbol = ''
+        sup_symbol = ''
 
     class Options(DefaultOptions):
         pass
@@ -93,11 +100,14 @@ class MarkdownConverter(object):
 
     def process_tag(self, node, convert_as_inline, children_only=False):
         text = ''
-        # markdown headings can't include block elements (elements w/newlines)
+
+        # markdown headings or cells can't include
+        # block elements (elements w/newlines)
         isHeading = html_heading_re.match(node.name) is not None
+        isCell = node.name in ['td', 'th']
         convert_children_as_inline = convert_as_inline
 
-        if not children_only and isHeading:
+        if not children_only and (isHeading or isCell):
             convert_children_as_inline = True
 
         # Remove whitespace-only textnodes in purely nested nodes
@@ -124,7 +134,7 @@ class MarkdownConverter(object):
 
         # Convert the children first
         for el in node.children:
-            if isinstance(el, Comment):
+            if isinstance(el, Comment) or isinstance(el, Doctype):
                 continue
             elif isinstance(el, NavigableString):
                 text += self.process_text(el)
@@ -139,22 +149,26 @@ class MarkdownConverter(object):
         return text
 
     def process_text(self, el):
-        text = six.text_type(el)
+        text = six.text_type(el) or ''
 
         # dont remove any whitespace when handling pre or code in pre
-        if (el.parent.name == 'pre'
-                or (el.parent.name == 'code' and el.parent.parent.name == 'pre')):
-            return escape(text or '')
+        if not (el.parent.name == 'pre'
+                or (el.parent.name == 'code'
+                    and el.parent.parent.name == 'pre')):
+            text = whitespace_re.sub(' ', text)
 
-        cleaned_text = escape(whitespace_re.sub(' ', text or ''))
+        if el.parent.name != 'code':
+            text = escape(text, self.options['escape_underscores'])
 
         # remove trailing whitespaces if any of the following condition is true:
         # - current text node is the last node in li
         # - current text node is followed by an embedded list
-        if el.parent.name == 'li' and (not el.next_sibling or el.next_sibling.name in ['ul', 'ol']):
-            return cleaned_text.rstrip()
+        if (el.parent.name == 'li'
+                and (not el.next_sibling
+                     or el.next_sibling.name in ['ul', 'ol'])):
+            text = text.rstrip()
 
-        return cleaned_text
+        return text
 
     def __getattr__(self, attr):
         # Handle headings
@@ -193,14 +207,17 @@ class MarkdownConverter(object):
         prefix, suffix, text = chomp(text)
         if not text:
             return ''
-        if convert_as_inline:
-            return text
         href = el.get('href')
         title = el.get('title')
         # For the replacement see #29: text nodes underscores are escaped
-        if self.options['autolinks'] and text.replace(r'\_', '_') == href and not title:
+        if (self.options['autolinks']
+                and text.replace(r'\_', '_') == href
+                and not title
+                and not self.options['default_title']):
             # Shortcut syntax
             return '<%s>' % href
+        if self.options['default_title'] and not title:
+            title = href
         title_part = ' "%s"' % title.replace('"', r'\"') if title else ''
         return '%s[%s](%s%s)%s' % (prefix, text, href, title_part, suffix) if href else text
 
@@ -301,7 +318,7 @@ class MarkdownConverter(object):
                 el = el.parent
             bullets = self.options['bullets']
             bullet = bullets[depth % len(bullets)]
-        return '%s %s\n' % (bullet, text or '')
+        return '%s %s\n' % (bullet, (text or '').strip())
 
     def convert_p(self, el, text, convert_as_inline):
         if convert_as_inline:
@@ -311,7 +328,7 @@ class MarkdownConverter(object):
     def convert_pre(self, el, text, convert_as_inline):
         if not text:
             return ''
-        return '\n```\n%s\n```\n' % text
+        return '\n```%s\n%s\n```\n' % (self.options['code_language'], text)
 
     convert_s = convert_del
 
@@ -319,6 +336,10 @@ class MarkdownConverter(object):
 
     convert_samp = convert_code
 
+    convert_sub = abstract_inline_conversion(lambda self: self.options['sub_symbol'])
+
+    convert_sup = abstract_inline_conversion(lambda self: self.options['sup_symbol'])
+
     def convert_table(self, el, text, convert_as_inline):
         return '\n\n' + text + '\n'
 
